#!/usr/bin/env python
# coding: utf-8

# Copyright (c) Dou Du,Casper Welzel Andersen.
# Distributed under the terms of the Modified BSD License.

version_info = (3, 1, 1)
__version__ = ".".join(map(str, version_info))
