"use strict";(self.webpackChunkwidget_periodictable=self.webpackChunkwidget_periodictable||[]).push([[480],{480:function(e,t,i){var r=this&&this.__createBinding||(Object.create?function(e,t,i,r){void 0===r&&(r=i),Object.defineProperty(e,r,{enumerable:!0,get:function(){return t[i]}})}:function(e,t,i,r){void 0===r&&(r=i),e[r]=t[i]}),n=this&&this.__setModuleDefault||(Object.create?function(e,t){Object.defineProperty(e,"default",{enumerable:!0,value:t})}:function(e,t){e.default=t}),u=this&&this.__importStar||function(e){if(e&&e.__esModule)return e;var t={};if(null!=e)for(var i in e)"default"!==i&&Object.prototype.hasOwnProperty.call(e,i)&&r(t,e,i);return n(t,e),t};Object.defineProperty(t,"__esModule",{value:!0});const a=i(829),o=u(i(367)),c=i(657),d={id:"widget_periodictable:plugin",requires:[a.IJupyterWidgetRegistry],activate:function(e,t){t.registerWidget({name:c.MODULE_NAME,version:c.MODULE_VERSION,exports:o})},autoStart:!0};t.default=d}}]);